#!/bin/bash
#

# build_numpy.sh 

# N.B. We don't actually need a qualifier for this build, but using the
# more modern compiler gives us better optimization.
usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9|e10>:<p271[01]> <prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

die()
{
    local exitval
    if [[ "$1" =~ ^[0-9]*$ ]]; then (( exitval = $1 )); shift; else (( exitval = 1 )); fi
    echo "ERROR: $@" 1>&2
    exit $exitval
}

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    for qual in "${qlist[@]}"; do
        case $qual in
            e9|e10)
                cqual=$qual
                ;;
            p271[01])
                pqual=$qual
                ;;
            *)
                die "Unrecognized qualifier $qual"
        esac
    done
    if [[ -z "$cqual" ]]; then
        die "Base qualifier is mandatory"
    fi
    if [[ -z "$pqual" ]] && (( "${cqual#e}" > 8 )); then
        die "A python qualifier is mandatory for e >= 9 (e.g. p2710)."
    fi
    case ${extraqual} in
        prof)
            :
            ;;
        debug|opt)
            die "${extraqual} builds of numpy are not supported at this time."
            ;;
        *)
            die "Unrecognized build qualifer ${extraqual}"
    esac
    fullqual=${cqual}${pqual:+:${pqual}}:${extraqual}
    qualdir=`echo ${fullqual} | sed -e 's/:/-/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

set_compiler_flags()
{
  case ${extraqual} in
    opt)
      cflags="-O3 -g -DNDEBUG"
      ;;
    debug)
      cflags="-g -O0"
      ;;
    prof)
      cflags="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
      ;;
  esac
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=numpy
origpkgver=v1_11_0
pkgver=${origpkgver}b
ssibuildshims_version=v0_10_00
setuptools_version=v22_0_5
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims
set_qualifiers
set_compiler_flags
get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "fake setup failed"

setup -B setuptools ${setuptools_version} ${pqual:+-q ${pqual}} -z ${product_dir}:${PRODUCTS} || \
  die "unable to set up UPS package for python setuptools ${setuptools_version} ${pqual:+-q ${pqual}}"

if [ ${OS1} = "Darwin" ]
then
   PATH=`dropit /usr/bin/get-cert $PATH`
fi

set -x

# work in a build directory, which will not be part of any distribution
python_shortdotversion=${PYTHON_VERSION#v}
python_shortdotversion=${python_shortdotversion//_/.}
python_shortdotversion=${python_shortdotversion%.*}

mkdir -p ${pkgdir}/lib/python${python_shortdotversion}/site-packages || die "failed to create ${pkgdir}/lib/python${python_shortdotversion}/site-packages"
mkdir -p ${blddir} || die "failed to create ${blddir}"

cd ${blddir} || die "Unable to cd into ${blddir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || die "Failed to untar ${pkgtarname}.tar.gz"
cd ${package}-${pkgdotver} || die "Failed to cd into ${package}-${pkgdotver}"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

FC=false \
CC=gcc \
CXX=false \
CFLAGS="${cflags}" \
python setup.py build -j ${ncore} install --prefix ${pkgdir} || die "python setup.py install failed"

# run some tests
nosetests -w ${pkgdir}/lib/python${python_shortdotversion}/site-packages/numpy*/numpy/tests || die "nosetests failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "numpy is installed at ${NUMPY_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
